@@ -39,13 +39,14 @@ KNOWN_KEYS = {"nelec", "spin", "hf_occ", "schedule", "maxiter",
               "mrrept2sd", "mrrept2-ijrs", "mrrept2-ij", "mrrept2-rs", "mrrept2-ijr",
               "mrrept2-rsi", "mrrept2-ir", "mrrept2-i", "mrrept2-r",
               "big_site", "stopt_dmrg", "stopt_compression", "stopt_sampling",
-              "model", "k_symmetry", "k_irrep", "k_mod", "init_mps_center",
+              "model", "k_symmetry", "k_irrep", "k_mod", "init_mps_center", "heisenberg",
               "use_complex", "real_density_matrix", "expt_algo_type", "one_body_parallel_rule",
               "davidson_max_iter", "davidson_soft_max_iter", "linear_soft_max_iter",
               "n_sub_sweeps", "complex_mps", "split_states", "trans_mps_to_complex",
               "use_general_spin", "trans_integral_to_spin_orbital", "store_wfn_spectra",
               "tran_bra_range", "tran_ket_range", "tran_triangular", "use_hybrid_complex",
-              "mem_ratio", "min_mpo_mem", "qc_mpo_type", "full_integral", "skip_inact_ext_sites"}
+              "mem_ratio", "min_mpo_mem", "qc_mpo_type", "full_integral", "skip_inact_ext_sites",
+              "proj_weights", "proj_mps_tags"}
 
 REORDER_KEYS = {"noreorder", "fiedler", "reorder", "gaopt", "nofiedler",
                 "irrep_reorder"}
@@ -143,6 +144,13 @@ def parse(fname):
     if len(schedule) == 0:
         schedule = get_schedule(dic)
         site_dependent_bdims = [[]] * len(schedule)
+    
+    if "memory," in dic:
+        dic["mem"] = dic["memory,"].replace(",", "")
+        del dic["memory,"]
+    elif "memory" in dic:
+        dic["mem"] = dic["memory"].replace(",", "")
+        del dic["memory"]
 
     tmp = list(zip(*schedule))
     nsweeps = np.diff(tmp[0]).tolist()
